/*
 * Copyright (C) 2011 ~ 2018 Deepin Technology Co., Ltd.
 *
 * Author:     sbw <sbw@sbw.so>
 *             kirigaya <kirigaya@mkacg.com>
 *             Hualet <mr.asianwang@gmail.com>
 *
 * Maintainer: sbw <sbw@sbw.so>
 *             kirigaya <kirigaya@mkacg.com>
 *             Hualet <mr.asianwang@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CONTROLWIDGET_H
#define CONTROLWIDGET_H

#include <dtkwidget_global.h>
#include <dtkcore_global.h>
#include "userinfo.h"

#include <DFloatingButton>
#include <DBlurEffectWidget>

#include <QWidget>
#include <QEvent>
#include <QMouseEvent>

namespace dss {
namespace module {
class BaseModuleInterface;
}
} // namespace dss

DWIDGET_USE_NAMESPACE

DCORE_BEGIN_NAMESPACE
class DConfig;
DCORE_END_NAMESPACE

class MediaWidget;
class QHBoxLayout;
class QPropertyAnimation;
class QLabel;
class QMenu;
class SessionBaseModel;
class KBLayoutListView;
class TipsWidget;
class TipContentWidget;
class SessionPopupWidget;
class UserListPopupWidget;
class RoundPopupWidget;

const int BlurRadius = 15;
const int BlurTransparency = 70;

class FlotingButton : public DFloatingButton
{
    Q_OBJECT
public:
    explicit FlotingButton(QWidget *parent = nullptr)
        : DFloatingButton(parent) {
        installEventFilter(this);
    }
    explicit FlotingButton(QStyle::StandardPixmap iconType = static_cast<QStyle::StandardPixmap>(-1), QWidget *parent = nullptr)
        : DFloatingButton(iconType, parent) {
        installEventFilter(this);
    }
    explicit FlotingButton(DStyle::StandardPixmap iconType = static_cast<DStyle::StandardPixmap>(-1), QWidget *parent = nullptr)
        : DFloatingButton(iconType, parent) {
        installEventFilter(this);
    }
    explicit FlotingButton(const QString &text, QWidget *parent = nullptr)
        : DFloatingButton(text, parent) {
        installEventFilter(this);
    }
    FlotingButton(const QIcon& icon, const QString &text = QString(), QWidget *parent = nullptr)
        : DFloatingButton(icon, text, parent) {
        installEventFilter(this);
    }

    void setTipText(const QString &tipText) {
        m_tipText = tipText;
    }
    QString tipText() const {
        return m_tipText;
    }

Q_SIGNALS:
    void requestShowMenu();
    void requestShowTips();
    void requestHideTips();

protected:
    bool eventFilter(QObject *watch, QEvent *event) override;

private:
    QString m_tipText;
};
class ControlWidget : public QWidget
{
    Q_OBJECT
public:
    explicit ControlWidget(const SessionBaseModel *model, QWidget *parent = nullptr);

    void setModel(const SessionBaseModel *model);
    void setUser(std::shared_ptr<User> user);

    void initKeyboardLayoutList();

signals:
    void requestSwitchUser(std::shared_ptr<User> user);
    void requestShutdown();
    void requestSwitchSession(const QString &session);
    void requestShowModule(const QString &name);

public slots:
    void addModule(dss::module::BaseModuleInterface *module);
    void setUserSwitchEnable(const bool visible);
    void setSessionSwitchEnable(const bool visible);
    void chooseToSession(const QString &session);
    void setKBLayoutVisible();
    void setKeyboardType(const QString& str);
    void setKeyboardList(const QStringList& str);
    void onItemClicked(const QString& str);
    void showSessionPopup();
    void showUserListPopupWidget();

protected:
    void keyReleaseEvent(QKeyEvent *event) Q_DECL_OVERRIDE;
    void showEvent(QShowEvent *event) Q_DECL_OVERRIDE;

private:
    void initUI();
    void initConnect();
    void updateLayout();
    void updateTapOrder();
    int focusedBtnIndex();
    void showPopupWidget(const FlotingButton *clickedBtn);

private slots:
    void showInfoTips();
    void hideInfoTips();

private:
    QList<FlotingButton *> m_showedBtnList;

    QHBoxLayout *m_mainLayout;
    FlotingButton *m_switchUserBtn;
    FlotingButton *m_powerBtn;
    FlotingButton *m_sessionBtn;
    FlotingButton *m_keyboardBtn;         // 键盘布局按钮

    std::shared_ptr<User> m_curUser;
    const SessionBaseModel *m_model;
    QList<QMetaObject::Connection> m_connectionList;

    QMenu *m_contextMenu;
    QMap<QString, QWidget *> m_modules;

    TipContentWidget *m_tipContentWidget;       // 显示按钮文字tip
    TipsWidget *m_tipsWidget;                   // 显示插件提供widget tip

    RoundPopupWidget *m_roundPopupWidget;       // 圆角弹窗
    KBLayoutListView *m_kbLayoutListView;       // 键盘布局列表
    SessionPopupWidget *m_sessionPopupWidget;   // session 列表
    UserListPopupWidget *m_userListPopupWidget; // 用户列表
};

#endif // CONTROLWIDGET_H
